<?php

namespace Drupal\pagedesigner_responsive_images;

use Drupal\Core\Render\Markup;
use Symfony\Component\Yaml\Yaml as YamlParser;
use Drupal\Component\Serialization\Yaml as YamlSerializer;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;
use Drupal\Core\File\FileUrlGenerator;

/**
 * Extend Drupal's Twig_Extension class.
 */
class ResponsiveImageStylesTwigExtension extends AbstractExtension {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file URL generator service.
   *
   * @var \Drupal\Core\File\FileUrlGenerator
   */
  protected $urlGenerator;

  /**
   * Creates an Responsive Image Styles Twig Extension.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\File\FileUrlGenerator $urlGenerator
   *   The file URL generator service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, FileUrlGenerator $urlGenerator) {
    $this->entityTypeManager = $entity_type_manager;
    $this->urlGenerator = $urlGenerator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'), $container->get('file_url_generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions() {
    return [
      new TwigFunction('image_style', [$this, 'getImageStyles']),
      new TwigFunction('styled_image_url', [$this, 'getStyledImageUrl']),
    ];
  }

  /**
   * Returns associative array from $string.
   *
   * @param array|object $sizes
   *   JSON encoded component sizes. Can be of mulitple types.
   * @param string $template
   *   Image style template.
   *
   * @return array
   *   Array containing the image styles.
   */
  public function getImageStyles($sizes, string $template) {

    if (\is_array($sizes) && array_key_exists('#text', $sizes) && is_string($sizes['#text'])) {
      $sizes = json_decode($sizes['#text'], TRUE, 512, JSON_THROW_ON_ERROR);
    }

    if (\is_object($sizes) && get_class($sizes) == Markup::class) {
      $sizes = json_decode($sizes->jsonSerialize(), TRUE, 512, JSON_THROW_ON_ERROR);
    }

    if (\is_string($sizes)) {
      $sizes = json_decode($sizes, TRUE, 512, JSON_THROW_ON_ERROR);
    }

    $config = $this->entityTypeManager->getStorage('image_style_template')->load($template);
    if ($config == NULL) {
      $config = $this->entityTypeManager->getStorage('image_style_template')->load('image_standard');
    }

    $imageStyles = [];
    $settings = YamlParser::parse(YamlSerializer::decode($config->settings));
    foreach ($settings as $breakpont => $templates) {
      $imageStyles[$breakpont] = [
        'templates' => $templates,
        'size'      => $sizes[$breakpont],
      ];
    }
    return $imageStyles;
  }

  /**
   * Return URL of a styled image.
   *
   * @param string $uri
   *   Image Uri.
   * @param string $styleName
   *   Image style.
   *
   * @return string
   *   URL to image after image style is applied.
   */
  public function getStyledImageUrl(string $uri, string $styleName = '') {
    if (empty($styleName)) {
      return $this->urlGenerator->transformRelative($this->urlGenerator->generateString($uri));
    }
    $style = $this->entityTypeManager->getStorage('image_style')->load($styleName);
    return $this->urlGenerator->transformRelative($style->buildUrl($uri));
  }

}
